import Vue from 'vue'

declare module 'vue/types/vue' {
  interface Vue {
    /**
     * 添加全局事件监听，可通过$emitBus触发，通过$offBus取消监听
     * @param event 事件名称，必须全局唯一，否则将覆盖掉同名事件
     * @param fn 事件回调
     */
    $onBus(event: string, fn: Function): this
    /**
     * 同$onBus，但在执行一次后自动取消监听
     * @param event 事件名称，必须全局唯一，否则将覆盖掉同名事件
     * @param fn 事件回调
     */
    $onceBus(event: string, fn: Function): this
    /**
     * 取消全局事件监听
     * @param event 事件名称
     * @param fn 事件回调
     */
    $offBus(event: string, fn: Function): this
    /**
     * 触发全局事件
     * @param event 事件名称
     */
    $emitBus(event: string): this
  }
}