export interface AxiosAdapterCacheConfig {
  maxAge?: number
}

export interface AxiosAdapterConfig {
  cache?: AxiosAdapterCacheConfig
}

export interface AxiosInterfaceErrorHandleConfig {
  handleError(err: Error): PromiseRejectionEvent
}

export interface AxiosInterfaceLoadingConfig {
  showLoader(): void
  hideLoader(): void
}

export interface AxiosInterfaceAuthorizationConfig {
  MAC_ID?: string
  MAC_KEY?: string
  decode?: boolean
  getMacKey(): string
}

export interface AxiosInterceptorConfig {
  applicationForm?: boolean
  dataToUnderline?: boolean
  dataToHump?: boolean
  responseDataToUnderline?: boolean
  responseDataToHump?: boolean
  responseErrToUnderline?: boolean
  responseErrToHump?: boolean
  disableCache?: boolean
  errorHandle?: boolean | AxiosInterfaceErrorHandleConfig
  loading?: boolean | AxiosInterfaceLoadingConfig
  authorization?: boolean | AxiosInterfaceAuthorizationConfig
  removeEmpty?: boolean
  returnResponseData?: boolean
}

export interface AxiosPluginConfig {
  defaults?: object
  adapters?: AxiosAdapterConfig
  interceptor?: AxiosInterceptorConfig
}

export interface BusPluginConfig {
  prefix?: string | boolean
}

export interface UtilAuthMacConfig {
  requestContentSymbol: string
  authMacSymbol: string
  nonceLen: number
}

export interface UtilFmtDateConfig {
  format?: string
}

export interface UtilFmtCurrencyConfig {
  places?: number
  symbol?: string
  thousand?: string
  decimal?: string
}

export interface UtilFmtNumberConfig {
  places?: number
  thousand?: string
  decimal?: string
}

export interface UtilFmtNumberLabelConfig {
  digitsLabel?: string[]
  digits?: number[]
  decimal?: number
}

export interface UtilFmtConfig {
  date?: boolean | UtilFmtDateConfig
  currency?: boolean | UtilFmtCurrencyConfig
  imgDomain?: boolean | string
  number?: boolean | UtilFmtNumberConfig
  numberLabel?: boolean | UtilFmtNumberLabelConfig
}

export interface UtilRandConfig {
  model?: string[]
}

export interface UtilImageView2ParamsConfig {
  interlace?: number
}

export interface UtilImageView2Config {
  name?: string
  params?: UtilImageView2ParamsConfig
}

export interface SpecialCharacter {
  char: string
  escape: string
  decimal: string
}

export interface SpecialCharacterFactory {
  getByChar(char: string): string
  getByEscape(escape: string): string
  getByDecimal(decimal: string): string
}

export interface HtmlSpecialCharacterRegEncodeFn {
  (args: string, specialCharacterFactory: SpecialCharacterFactory): any
}

export interface HtmlSpecialCharacterRegDecodeFn {
  (args: string, specialCharacterFactory: SpecialCharacterFactory): any
}

export interface HtmlSpecialCharacter {
  REG_HTML_ENCODE?: RegExp
  REG_HTML_DECODE?: RegExp
  REG_HTML_DECODE_DATA?: SpecialCharacter[]
  regEncodeFn: HtmlSpecialCharacterRegEncodeFn
  regDecodeFn: HtmlSpecialCharacterRegDecodeFn
}

export interface UtilHtmlConfig {
  htmlSpecialCharacter: HtmlSpecialCharacter
}


export interface FilterConfig {
  date?: boolean
  currency?: boolean
  value?: boolean
  url?: boolean
  imageView2?: boolean
  number?: boolean
  numberLabel?: boolean
}

export interface PluginConfig {
  axios?: boolean | AxiosPluginConfig
  bus?: boolean | BusPluginConfig
}

export interface UtilConfig {
  authMac?: boolean | UtilAuthMacConfig
  fmt?: boolean | UtilFmtConfig
  rand?: boolean | UtilRandConfig
  imageView2?: boolean | UtilImageView2Config
  html?: boolean | UtilHtmlConfig
}

export interface VueAssistConfig {
  debug?: boolean
  filters?: boolean | FilterConfig
  plugins?: boolean | PluginConfig
  utils?: boolean | UtilConfig
  directives?: boolean | object
}

export default VueAssistConfig
